/**
 * Copyright (c) 2020 xxx Inc.
 * File              : preprocess.h
 * Author            : 
 * Date              : 2020-05-07
 * Last Modified Date: 2020-05-07
 * Last Modified By  : 
 */
#ifndef VIDEOFLOW_DL_PREPROCESS_H__
#define VIDEOFLOW_DL_PREPROCESS_H__

#include <map>
#include <vector>

//
#include <vf/devices/cuda.h>
#include <vf/parallel.h>

namespace vf {
namespace dl {

enum PreProcessType { kCopy, kScaleMeanStd };

/**
 * \brief  preprocess function type
 */
struct PreProcessFuncType {
  typedef void (*FType)(TStream stream, int N, const std::vector<int>& shape,
                        const float* src, float* dst, float alpha,
                        const float* beta, const float* gamma);
};

template <int KDevice, int PixFmt, int PType>
struct PreProcessFunc;

template <int KDevice>
struct PreProcessFuncFactory {
  /**
   * \brief  static register factory
   */
  static std::map<int, std::map<int, PreProcessFuncType::FType>>&
  kRegisteredFuncs();

  static PreProcessFuncType::FType Get(int pix_fmt, int preprocess) {
    auto iter1 =
        PreProcessFuncFactory<KDevice>::kRegisteredFuncs().find(pix_fmt);
    if (PreProcessFuncFactory<KDevice>::kRegisteredFuncs().end() == iter1) {
      return nullptr;
    }
    auto iter2 = iter1->second.find(preprocess);
    if (iter1->second.end() == iter2) {
      return nullptr;
    }
    return iter2->second;
  }
};

template <int KDevice>
std::map<int, std::map<int, PreProcessFuncType::FType>>&
PreProcessFuncFactory<KDevice>::kRegisteredFuncs() {
  static std::map<int, std::map<int, PreProcessFuncType::FType>>
      __kRegisteredFuncs;
  return __kRegisteredFuncs;
}

template <int KDevice>
struct PreProcessFuncRegistry {
  PreProcessFuncRegistry(int pix_fmt, int preprocess,
                         PreProcessFuncType::FType func) {
    PreProcessFuncFactory<KDevice>::kRegisteredFuncs()[pix_fmt][preprocess] =
        func;
  }
};

#define RegisterPreProcessFunc(KDevice, PixFmt, rPixFmt, PreProcess) \
  PreProcessFuncRegistry<KDevice>                                    \
      __PreProcessFunc__##KDevice##_##PixFmt##_##PreProcess##_(      \
          PixFmt, PreProcess,                                        \
          PreProcessFunc<KDevice, rPixFmt, PreProcess>::Map);

}  // namespace dl
}  // namespace vf
#endif
