/**
 * Copyright (c) 2020 xxx Inc.
 * File              : image-decoder.h
 * Author            : 
 * Date              : 2020-05-07
 * Last Modified Date: 2020-05-07
 * Last Modified By  : 
 */
#ifndef VIDEOFLOW_FLOW_IMAGE_DECODER_H__
#define VIDEOFLOW_FLOW_IMAGE_DECODER_H__

#include <cxxutil/block-queue.h>
#include <vf/flow/op.h>
#include <vf/vf-frame.h>

#include <opencv2/opencv.hpp>

namespace vf {
namespace flow {

struct ImageDecoderParam : public cxxutil::Parameter<ImageDecoderParam> {
  int width;
  int height;
  CXXUTIL_DECLARE_PARAMETER(ImageDecoderParam) {
    CXXUTIL_DECLARE_FIELD(width).set_default(-1).describe("decode image width");
    CXXUTIL_DECLARE_FIELD(height).set_default(-1).describe(
        "decode image height");
  }
};

class ImageTask {
 public:
  void Wait() {
    monitor_.lock();
    if (finished_ == false) {
      monitor_.wait();
    }
    monitor_.unlock();
  }

  void Reset() {
    monitor_.lock();
    finished_ = false;
    monitor_.unlock();
  }

  void OnFinish() {
    monitor_.lock();
    finished_ = true;
    monitor_.notify();
    monitor_.unlock();
  }

 public:
  VFFrame frame;
  cv::Mat img;
  void *extra;
  std::string result;

 protected:
  bool finished_ = false;
  cxxutil::Monitor monitor_;
};

class ImageDecoder : public Entry {
 public:
  ImageDecoder(const cxxutil::ParamType &params = {});
  virtual ~ImageDecoder();

  // inherited from op
  int Init(int max_orders) override;
  void Interrupt() override;

  // inherited from entry
 protected:
  DataType run(const DataType &res_data) override;
  void ReleaseOutput(DataType &output) override;

  // self-defined
 public:
  /**
   * \brief Put new image data
   *
   * \param img_stream image stream
   * \param stream_len length of the stream
   * \param channel_id image channel
   * \param timestamp image timestamp
   * \return int status code
   */
  virtual std::shared_ptr<ImageTask> PutImage(const uint8_t *img_stream,
                                              int64_t stream_len,
                                              const std::string &channel_id,
                                              int64_t timestamp);

 protected:
  /**
   * \brief  process image task asynchronously
   *
   * \param task task to be processed
   *
   * \return order of the task to be processed
   */
  int ProcessImageTaskAsync(ImageTask *task);

 protected:
  ImageDecoderParam param_;
  cxxutil::BlockQueue<ImageTask *> *tasks_ = nullptr;
};

}  // namespace flow
}  // namespace vf
#endif
