/**
 * Copyright (c) 2020 xxx Inc.
 * File              : video-decoder.h
 * Author            : 
 * Date              : 2020-05-07
 * Last Modified Date: 2020-05-07
 * Last Modified By  : 
 */
#ifndef VIDEOFLOW_FLOW_VIDEO_DECODER_H__
#define VIDEOFLOW_FLOW_VIDEO_DECODER_H__

#include <vf/flow/op.h>
#include <vf/io/video-decoder.h>

namespace vf {
namespace flow {

struct VideoDecodeParam : public cxxutil::Parameter<VideoDecodeParam> {
  int width;
  int height;
  int sample_rate;
  int64_t timestamp_interval;
  bool with_mvs;
  int pixel_format;
  int decoder;
  CXXUTIL_DECLARE_PARAMETER(VideoDecodeParam) {
    CXXUTIL_DECLARE_FIELD(width).set_default(-1).describe("decode frame width");
    CXXUTIL_DECLARE_FIELD(height).set_default(-1).describe(
        "decode frame height");
    CXXUTIL_DECLARE_FIELD(sample_rate)
        .set_default(1)
        .describe("frame sample rate");
    CXXUTIL_DECLARE_FIELD(timestamp_interval)
        .set_default(0)
        .describe("timestamp interval in milliseconds");
    CXXUTIL_DECLARE_FIELD(with_mvs).set_default(false).describe(
        "with motion vectors");
    CXXUTIL_DECLARE_FIELD(pixel_format)
        .add_enum("GRAY", VF_PIX_FMT_GRAY)
        .add_enum("BGR", VF_PIX_FMT_BGR)
        .add_enum("RGB", VF_PIX_FMT_RGB)
        .add_enum("YUV420P", VF_PIX_FMT_YUV420P)
        .add_enum("NV12", VF_PIX_FMT_NV12)
        .add_enum("NV21", VF_PIX_FMT_NV21)
        .describe("frame pixel format");
    CXXUTIL_DECLARE_FIELD(decoder)
        .set_default(kAuto)
        .add_enum("Auto", kAuto)
        .add_enum("CPU", kCPU)
        .add_enum("GPU", kGPU)
        .add_enum("Xavier", kXavier)
        .add_enum("MLU", kMLU)
        .describe("frame pixel format");
  }
};

class VideoDecoder : public StreamTaskEntry {
 public:
  VideoDecoder(
      const std::vector<std::pair<std::string, std::string>> &params = {});
  virtual ~VideoDecoder();

  int Open(const std::vector<std::string> &params) override;

  /**
   * \brief Put new video data
   *
   * \param timestamp timestamp of the data
   * \param data_stream data stream
   * \param stream_len length of the stream
   * \param extra other extra information
   * \return int status code
   */
  int PutData(int64_t timestamp, uint8_t *data_stream, int64_t stream_len,
              void *extra = nullptr) override;

 public:
  void Interrupt() override;

 protected:
  DataType run(const DataType &res_data) override;

  std::vector<void *> MallocResource() override;
  void FreeResource(std::vector<void *> output) override;

 public:
  vf::io::VideoDecoder *decoder() { return decoder_; }

 protected:
  vf::io::VideoDecoder *decoder_ = nullptr;
  VideoDecodeParam param_;
  bool first_frame_ = true;
  TStream stream_ = nullptr;
  Context ctx_;

  std::atomic<bool> should_stop_;
};

}  // namespace flow
}  // namespace vf
#endif
