/**
 * Copyright (c) 2020 Inc.
 * File              : rgb.cc
 * Author            :
 * Date              : 2020-05-07
 * Last Modified Date: 2020-05-07
 * Last Modified By  :
 */
#include "rgb.h"

namespace vf {

template <typename T1, typename T2, int sPixFmt, int dPixFmt>
struct PixelConvert<kCPU, T1, T2, sPixFmt, dPixFmt> {
  inline static void Map(TStream stream, const PixelDescr<T1>* src, int dst_N,
                         int dst_h, int dst_w, const CoordMapParam* mparams,
                         PixelDescr<T2> const** dsts) {
    const int omp_threads = 0 == omp_num_threads ? 1 : omp_num_threads;
    for (int dst_idx = 0; dst_idx < dst_N; ++dst_idx) {
      const PixelDescr<T2>* dst = dsts[dst_idx];
      const int h_step = (dst->h + omp_threads - 1) / omp_threads;
#if USE_OpenMP
#pragma omp parallel for num_threads(omp_threads)
#endif
      for (int y = 0; y < dst->h; y += h_step) {
        int h_end = y + h_step;
        if (h_end > dst->h) h_end = dst->h;
        for (int dy = y; dy < h_end; ++dy) {
          for (int dx = 0; dx < dst->w; ++dx) {
            RGB2RGBCvtFunc(dst_idx, dx, dy, src, mparams + dst_idx,
                           dsts[dst_idx], isswap(sPixFmt, dPixFmt));
          }
        }
      }
    }
  }
};

// bgr source images
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGR, VF_PIX_FMT_BGR);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGR, VF_PIX_FMT_RGB);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGR, VF_PIX_FMT_BGRPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGR, VF_PIX_FMT_RGBPlanar);

RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGR, VF_PIX_FMT_BGRA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGR, VF_PIX_FMT_RGBA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGR, VF_PIX_FMT_BGRAPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGR, VF_PIX_FMT_RGBAPlanar);

RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRA, VF_PIX_FMT_BGRA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRA, VF_PIX_FMT_RGBA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRA, VF_PIX_FMT_BGRAPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRA, VF_PIX_FMT_RGBAPlanar);

// rgb source images
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGB, VF_PIX_FMT_BGR);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGB, VF_PIX_FMT_RGB);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGB, VF_PIX_FMT_BGRPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGB, VF_PIX_FMT_RGBPlanar);

RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGB, VF_PIX_FMT_BGRA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGB, VF_PIX_FMT_RGBA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGB, VF_PIX_FMT_BGRAPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGB, VF_PIX_FMT_RGBAPlanar);

RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBA, VF_PIX_FMT_BGRA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBA, VF_PIX_FMT_RGBA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBA, VF_PIX_FMT_BGRAPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBA, VF_PIX_FMT_RGBAPlanar);

// bgr planar source images
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRPlanar, VF_PIX_FMT_BGR);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRPlanar, VF_PIX_FMT_RGB);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRPlanar, VF_PIX_FMT_BGRPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRPlanar, VF_PIX_FMT_RGBPlanar);

RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRPlanar, VF_PIX_FMT_BGRA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRPlanar, VF_PIX_FMT_RGBA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRPlanar, VF_PIX_FMT_BGRAPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRPlanar, VF_PIX_FMT_RGBAPlanar);

RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRAPlanar, VF_PIX_FMT_BGRA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRAPlanar, VF_PIX_FMT_RGBA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRAPlanar, VF_PIX_FMT_BGRAPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_BGRAPlanar, VF_PIX_FMT_RGBAPlanar);

// rgb planar source images
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBPlanar, VF_PIX_FMT_BGR);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBPlanar, VF_PIX_FMT_RGB);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBPlanar, VF_PIX_FMT_BGRPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBPlanar, VF_PIX_FMT_RGBPlanar);

RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBPlanar, VF_PIX_FMT_BGRA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBPlanar, VF_PIX_FMT_RGBA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBPlanar, VF_PIX_FMT_BGRAPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBPlanar, VF_PIX_FMT_RGBAPlanar);

RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBAPlanar, VF_PIX_FMT_BGRA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBAPlanar, VF_PIX_FMT_RGBA);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBAPlanar, VF_PIX_FMT_BGRAPlanar);
RegisterRGBCvtFunc(kCPU, VF_PIX_FMT_RGBAPlanar, VF_PIX_FMT_RGBAPlanar);

}  // namespace vf
