/**
 * Copyright (c) 2020 xxx Inc.
 * File              : rgb.h
 * Author            : 
 * Date              : 2020-05-07
 * Last Modified Date: 2020-05-07
 * Last Modified By  : 
 */
#include "vf/pixcvt.h"

namespace vf {

XINLINE bool isswap(int s_pixfmt, int d_pixfmt) {
  if ((s_pixfmt == d_pixfmt) ||
      (s_pixfmt == VF_PIX_FMT_BGR && d_pixfmt == VF_PIX_FMT_BGRPlanar) ||
      (s_pixfmt == VF_PIX_FMT_RGB && d_pixfmt == VF_PIX_FMT_RGBPlanar) ||
      (s_pixfmt == VF_PIX_FMT_BGRPlanar && d_pixfmt == VF_PIX_FMT_BGR) ||
      (s_pixfmt == VF_PIX_FMT_RGBPlanar && d_pixfmt == VF_PIX_FMT_RGB)) {
    return false;
  }
  return true;
}

template <typename T1, typename T2>
XINLINE void RGB2RGBCvtFunc(int dst_idx, int dx, int dy,
                            const PixelDescr<T1>* src,
                            const CoordMapParam* mparam,
                            const PixelDescr<T2>* dst, bool swap) {
  // coordinates in the source image
  float sxf = dx * mparam->fx + mparam->sx;
  float syf = dy * mparam->fy + mparam->sy;
  int sx = int(sxf), sy = int(syf);

  // interpolation coefficients
  float alpha = sxf - sx, beta = syf - sy;

  int sx1y1, sx1y2, sx2y1, sx2y2;
  for (int c = 0; c < src->channels; ++c) {
    sx1y1 = sy * src->step[c] + int(sx * src->pstep[c]);
    if (sx + 1 < src->w) {
      sx2y1 = sx1y1 + int(src->pstep[c]);
    } else {
      sx2y1 = sx1y1;
    }

    if (sy + 1 < src->h) {
      sx1y2 = sx1y1 + src->step[c];
      sx2y2 = sx2y1 + src->step[c];
    } else {
      sx1y2 = sx1y1;
      sx2y2 = sx2y1;
    }
    const T1* src_ptr = src->data[c];
    int dc = swap ? 2 - c : c;
    T2* dst_ptr =
        (T2*)(dst->data[dc]) + dy * dst->step[dc] + int(dx * dst->pstep[dc]);

    float val1 = (1 - alpha) * src_ptr[sx1y1] + alpha * src_ptr[sx2y1];
    float val2 = (1 - alpha) * src_ptr[sx1y2] + alpha * src_ptr[sx2y2];
    *dst_ptr = T2((1 - beta) * val1 + beta * val2);
  }

  for (int c = src->channels; c < dst->channels; ++c) {
    T2* dptr =
        (T2*)(dst->data[c]) + dy * dst->step[c] + int(dx * dst->pstep[c]);
    *dptr = 0;
  }
}

#define RegisterRGBCvtFunc(KDevice, sPixFmt, dPixFmt)                \
  RegisterPixelCvtFunc(KDevice, uint8_t, uint8_t, sPixFmt, dPixFmt); \
  RegisterPixelCvtFunc(KDevice, uint8_t, float, sPixFmt, dPixFmt);   \
  RegisterPixelCvtFunc(KDevice, float, uint8_t, sPixFmt, dPixFmt);   \
  RegisterPixelCvtFunc(KDevice, float, float, sPixFmt, dPixFmt);

}  // namespace vf
