/**
 * Copyright (c) 2020 xxx Inc.
 * File              : yuv.cc
 * Author            : 
 * Date              : 2020-05-07
 * Last Modified Date: 2020-05-07
 * Last Modified By  : 
 */
#include "yuv.h"

namespace vf {

template <typename T1, typename T2, int sPixFmt, int dPixFmt>
struct PixelConvert<kCPU, T1, T2, sPixFmt, dPixFmt> {
  inline static void Map(TStream stream, const PixelDescr<T1>* src, int dst_N,
                         int dst_h, int dst_w, const CoordMapParam* mparams,
                         PixelDescr<T2> const** dsts) {
    const int omp_threads = 0 == omp_num_threads ? 1 : omp_num_threads;
    for (int dst_idx = 0; dst_idx < dst_N; ++dst_idx) {
      const PixelDescr<T2>* dst = dsts[dst_idx];
      const int h_step = (dst->h + omp_threads - 1) / omp_threads;
#if USE_OpenMP
#pragma omp parallel for num_threads(omp_threads)
#endif
      for (int y = 0; y < dst->h; y += h_step) {
        int h_end = y + h_step;
        if (h_end > dst->h) h_end = dst->h;
        for (int dy = y; dy < h_end; ++dy) {
          for (int dx = 0; dx < dst->w; ++dx) {
            YUV2RGBConvFunc<T2, sPixFmt>(dst_idx, dx, dy, src,
                                         mparams + dst_idx, dsts[dst_idx],
                                         isbgr(dPixFmt));
          }
        }
      }
    }
  }
};

// yuv420p - bgr source images
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_YUV420P, VF_PIX_FMT_BGR);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_YUV420P, VF_PIX_FMT_RGB);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_YUV420P, VF_PIX_FMT_BGRPlanar);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_YUV420P, VF_PIX_FMT_RGBPlanar);

// yuv420p - bgra source images
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_YUV420P, VF_PIX_FMT_BGRA);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_YUV420P, VF_PIX_FMT_RGBA);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_YUV420P, VF_PIX_FMT_BGRAPlanar);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_YUV420P, VF_PIX_FMT_RGBAPlanar);

// nv12 - bgr source images
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV12, VF_PIX_FMT_BGR);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV12, VF_PIX_FMT_RGB);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV12, VF_PIX_FMT_BGRPlanar);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV12, VF_PIX_FMT_RGBPlanar);

// nv12 - bgra source images
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV12, VF_PIX_FMT_BGRA);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV12, VF_PIX_FMT_RGBA);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV12, VF_PIX_FMT_BGRAPlanar);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV12, VF_PIX_FMT_RGBAPlanar);

// nv21 - bgr planar source images
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV21, VF_PIX_FMT_BGR);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV21, VF_PIX_FMT_RGB);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV21, VF_PIX_FMT_BGRPlanar);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV21, VF_PIX_FMT_RGBPlanar);

// nv21 - bgra planar source images
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV21, VF_PIX_FMT_BGRA);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV21, VF_PIX_FMT_RGBA);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV21, VF_PIX_FMT_BGRAPlanar);
RegisterYUVCvtFunc(kCPU, VF_PIX_FMT_NV21, VF_PIX_FMT_RGBAPlanar);

}  // namespace vf
