/**
 * Copyright (c) 2020 xxx Inc.
 * File              : rocketmq-connector.h
 * Author            : 
 * Date              : 2020-05-07
 * Last Modified Date: 2020-05-07
 * Last Modified By  : 
 */
#ifndef VIDEOFLOW_RESOURCE_ROCKETMQ_CONNECTOR_H__
#define VIDEOFLOW_RESOURCE_ROCKETMQ_CONNECTOR_H__

#include <vf/vf-def.h>

#ifdef USE_RocketMQ

#include <rocketmq/Arg_helper.h>
#include <rocketmq/DefaultMQProducer.h>

#include <cxxutil/mutex.h>
#include <cxxutil/thread.h>

#include <vf/resource/resource.h>

namespace vf {
namespace flow {

struct RocketMQConnectorParam
    : public cxxutil::Parameter<RocketMQConnectorParam> {
  std::string namesrv;
  std::string group;

  int buf_size;
  int64_t max_latency;

  std::string access_key;
  std::string sceret_key;
  std::string ons_channel;

  CXXUTIL_DECLARE_PARAMETER(RocketMQConnectorParam) {
    CXXUTIL_DECLARE_FIELD(namesrv).describe("rocketmq name server");
    CXXUTIL_DECLARE_FIELD(group).describe("rocketmq group");
    CXXUTIL_DECLARE_FIELD(buf_size)
        .set_lower_bound(1)
        .set_default(100)
        .describe("rocketmq send buffer size");
    CXXUTIL_DECLARE_FIELD(max_latency)
        .set_lower_bound(1)
        .set_default(500)
        .describe("max rocketmq send lantecy in ms");

    CXXUTIL_DECLARE_FIELD(access_key)
        .set_default("")
        .describe("rocketmq access key");
    CXXUTIL_DECLARE_FIELD(sceret_key)
        .set_default("")
        .describe("rocketmq sceret key");
    CXXUTIL_DECLARE_FIELD(ons_channel)
        .set_default("videoflow")
        .describe("rocketmq ons channel");
  }
};

/**
 * \brief  elastic search connector
 */
class RocketMQConnector : public flow::Resource {
 public:
  virtual ~RocketMQConnector();

  int Init(const std::string &name,
           const cxxutil::ParamType &params = {}) override;

  int WriteMessage(const std::string &topic, const std::string &msg);

 protected:
  friend class SendMessageTask;
  RocketMQConnectorParam param_;
  std::shared_ptr<rocketmq::DefaultMQProducer> producer_;
  std::vector<rocketmq::MQMessage> msgs_;
  int cur_msg_num_ = 0;
  cxxutil::ThreadTask *send_task_ = nullptr;
  cxxutil::Monitor timer_;
};

}  // namespace flow
}  // namespace vf
#endif
#endif
